@@ -12,12 +12,21 @@ import os.path
 import queue
 import subprocess
 
+
 def start_headless(mm_app_path, config_file, core_log_path=None, buffer_size_mb=1024):
     # TODO: add docstring
 
-    jar_directory = mm_app_path + '/plugins/micro-manager'
-    #TODO: how to determine right java version?
-    subprocess.Popen(['java', '-classpath', '"{}/*"'.format(jar_directory), 'org.micromanager.remote.HeadlessLauncher'])
+    # TODO: argument for java memory
+    jar_directory = mm_app_path + "/plugins/micro-manager"
+    # TODO: how to determine right java version?
+    subprocess.Popen(
+        [
+            "java",
+            "-classpath",
+            '"{}/*"'.format(jar_directory),
+            "org.micromanager.remote.HeadlessLauncher",
+        ]
+    )
 
     bridge = Bridge()
     core = bridge.get_core()
@@ -35,6 +44,7 @@ def start_headless(mm_app_path, config_file, core_log_path=None, buffer_size_mb=
 
 ### These functions outside class to prevent problems with pickling when running them in differnet process
 
+
 def _event_sending_fn(event_port, event_queue, debug=False):
     """
 
@@ -246,6 +256,7 @@ class Acquisition(object):
         post_hardware_hook_fn=None,
         post_camera_hook_fn=None,
         show_display=True,
+        storage_monitor_callback_fn=None,
         tile_overlap=None,
         max_multi_res_index=None,
         magellan_acq_index=None,
@@ -305,6 +316,9 @@ class Acquisition(object):
             calculated and updated from data
         show_display : bool
             show the image viewer window
+        storage_monitor_callback_fn : Callable
+            function that takes one argument (the Axes of the image that just finished saving) and gets called
+            whenever a new image is writtent to disk
         magellan_acq_index : int
             run this acquisition using the settings specified at this position in the main
             GUI of micro-magellan (micro-manager plugin). This index starts at 0
@@ -427,6 +441,12 @@ class Acquisition(object):
             )
             self.event_process.start()
 
+        if storage_monitor_callback_fn is not None:
+            self._dataset = Dataset(remote_storage_monitor=self._remote_acq.get_storage_monitor())
+            self._dataset._add_storage_monitor_fn(
+                callback_fn=storage_monitor_callback_fn, debug=self._debug
+            )
+
     def __enter__(self):
         return self
 
@@ -437,12 +457,6 @@ class Acquisition(object):
         # now wait on it to finish
         self.await_completion()
 
-    def get_disk_location(self):
-        """
-        Return the path where the dataset is on disk
-        """
-        return self._dataset_disk_location
-
     def get_dataset(self):
         """
         Get access to the dataset backing this acquisition. If the acquisition is in progress,
@@ -450,11 +464,13 @@ class Acquisition(object):
         load the dataset from disk on the Python side for better performance
         """
         if self._finished:
-            if self._dataset is None or self._dataset._remote_storage is not None:
+            if self._dataset is None or self._dataset._remote_storage_monitor is not None:
                 self._dataset = Dataset(self._dataset_disk_location)
         elif self._dataset is None:
             # Load remote storage
-            self._dataset = Dataset(remote_storage=self._remote_acq.get_storage())
+            self._dataset = Dataset(remote_storage_monitor=self._remote_acq.get_storage_monitor())
+            # Monitor image arrival so they can be loaded on python side, but with no callback function
+            self._dataset._add_storage_monitor_fn(callback_fn=None, debug=self._debug)
 
         return self._dataset
 
