@@ -1,4 +1,3 @@
-
 """
 Created on Mon Dec  3 17:53:24 2018
 
@@ -24,8 +23,9 @@ class Position1d:
         z: The position
         zStage: Then name of the translation stage
     """
-    z: float    
-    stageName: str = ''
+
+    z: float
+    stageName: str = ""
     numAxes: int = 1
 
     def __post_init__(self):
@@ -37,14 +37,15 @@ class Position1d:
         return Position1d(**d)
 
     def toPropertyMap(self) -> PropertyMap:
-        return PropertyMap({"Device": Property(self.stageName),
-                "Position_um": PropertyArray([Property(self.z)])})
+        return PropertyMap(
+            {"Device": Property(self.stageName), "Position_um": PropertyArray([Property(self.z)])}
+        )
 
     @staticmethod
     def fromPropertyMap(pmap: PropertyMap) -> Position1d:
-        if len(pmap['Position_um']) != 1:
+        if len(pmap["Position_um"]) != 1:
             raise Exception("RERAR")
-        return Position1d(z=pmap['Position_um'][0].value, stageName=pmap['Device'].value)
+        return Position1d(z=pmap["Position_um"][0].value, stageName=pmap["Device"].value)
 
     def __repr__(self):
         return f"Position1d({self.stageName}, {self.z})"
@@ -59,9 +60,10 @@ class Position2d:
         y: The y position
         xyStage: The name of the 2 dimensional translation stage
     """
+
     x: float
     y: float
-    stageName: str = ''
+    stageName: str = ""
     numAxes: int = 2
 
     def __post_init__(self):
@@ -74,16 +76,19 @@ class Position2d:
         return Position2d(**d)
 
     def toPropertyMap(self) -> PropertyMap:
-        return PropertyMap({"Device": Property(self.stageName),
-                        "Position_um": PropertyArray([Property(self.x), Property(self.y)])})
+        return PropertyMap(
+            {
+                "Device": Property(self.stageName),
+                "Position_um": PropertyArray([Property(self.x), Property(self.y)]),
+            }
+        )
 
     @staticmethod
     def fromPropertyMap(pmap: PropertyMap) -> Position2d:
-        if len(pmap['Position_um'])!=2:
+        if len(pmap["Position_um"]) != 2:
             raise Exception("Errr")
-        x, y = pmap['Position_um'][0].value, pmap['Position_um'][1].value
-        return Position2d(x=x, y=y, stageName=pmap['Device'].value)
-
+        x, y = pmap["Position_um"][0].value, pmap["Position_um"][1].value
+        return Position2d(x=x, y=y, stageName=pmap["Device"].value)
 
     def mirrorX(self) -> Position2d:
         self.x *= -1
@@ -99,19 +104,21 @@ class Position2d:
     def __repr__(self):
         return f"Position2d({self.stageName}, {self.x}, {self.y})"
 
-    def __add__(self, other: Union[PositionList, Position2d, MultiStagePosition]) -> Union[PositionList, Position2d, MultiStagePosition]:
+    def __add__(
+        self, other: Union[PositionList, Position2d, MultiStagePosition]
+    ) -> Union[PositionList, Position2d, MultiStagePosition]:
         if isinstance(other, PositionList):
             return other.__add__(self)
         elif isinstance(other, MultiStagePosition):
             return other.__add__(self)
         elif isinstance(other, Position2d):
-            return Position2d(self.x + other.x,
-                              self.y + other.y,
-                              self.stageName)
+            return Position2d(self.x + other.x, self.y + other.y, self.stageName)
         else:
             raise TypeError(f"Type {type(other)} is not supported.")
 
-    def __sub__(self, other: Union[PositionList, Position2d, MultiStagePosition]) -> Union[PositionList, Position2d, MultiStagePosition]:
+    def __sub__(
+        self, other: Union[PositionList, Position2d, MultiStagePosition]
+    ) -> Union[PositionList, Position2d, MultiStagePosition]:
         if isinstance(other, PositionList):
             return other.copy().mirrorX().mirrorY().__add__(self)  # a-b == -b + a
         elif isinstance(other, MultiStagePosition):
@@ -119,16 +126,12 @@ class Position2d:
             other.getXYPosition().mirrorX().mirrorY()  # invert the object.
             return other.__add__(self)  # a-b == -b + a
         elif isinstance(other, Position2d):
-            return Position2d(self.x - other.x,
-                              self.y - other.y,
-                              self.stageName)
+            return Position2d(self.x - other.x, self.y - other.y, self.stageName)
         else:
             raise TypeError(f"Type {type(other)} is not supported.")
 
-    def __eq__(self, other: 'Position2d'):
-        return all([self.x == other.x,
-                    self.y == other.y,
-                    self.stageName == other.stageName])
+    def __eq__(self, other: "Position2d"):
+        return all([self.x == other.x, self.y == other.y, self.stageName == other.stageName])
 
 
 @dataclass
@@ -143,6 +146,7 @@ class MultiStagePosition:
         zStage: The name of the 1D stage
         positions: A list of `Position1d` and `Position2D` objects, usually just one of each.
     """
+
     label: str
     defaultXYStage: str
     defaultZStage: str
@@ -153,49 +157,59 @@ class MultiStagePosition:
     @staticmethod
     def fromDict(d: dict) -> MultiStagePosition:
         sps = []
-        for i in d['stagePositions']:
-            if i['numAxes'] == 1:
+        for i in d["stagePositions"]:
+            if i["numAxes"] == 1:
                 sps.append(Position1d.fromDict(i))
-            elif i['numAxes'] == 2:
+            elif i["numAxes"] == 2:
                 sps.append(Position2d.fromDict(i))
             else:
                 raise Exception()
-        return MultiStagePosition(label=d['label'],
-                                  defaultXYStage=d['defaultXYStage'],
-                                  defaultZStage=d['defaultZStage'],
-                                  stagePositions=sps,
-                                  gridRow=d['gridRow'],
-                                  gridCol=d['gridCol'])
+        return MultiStagePosition(
+            label=d["label"],
+            defaultXYStage=d["defaultXYStage"],
+            defaultZStage=d["defaultZStage"],
+            stagePositions=sps,
+            gridRow=d["gridRow"],
+            gridCol=d["gridCol"],
+        )
 
     def toPropertyMap(self) -> PropertyMap:
-        return PropertyMap({
-            "DefaultXYStage": Property(self.defaultXYStage),
-            "DefaultZStage": Property(self.defaultZStage),
-            "DevicePositions": PropertyMapArray([i.toPropertyMap() for i in self.stagePositions]),
-            "GridCol": Property(0),
-            "GridRow": Property(0),
-            "Label": Property(self.label),
-            "Properties": PropertyMap({})
-        })
-   
+        return PropertyMap(
+            {
+                "DefaultXYStage": Property(self.defaultXYStage),
+                "DefaultZStage": Property(self.defaultZStage),
+                "DevicePositions": PropertyMapArray(
+                    [i.toPropertyMap() for i in self.stagePositions]
+                ),
+                "GridCol": Property(0),
+                "GridRow": Property(0),
+                "Label": Property(self.label),
+                "Properties": PropertyMap({}),
+            }
+        )
 
     @staticmethod
     def fromPropertyMap(d: PropertyMap) -> MultiStagePosition:
         positions = []
-        for i in d['DevicePositions']:
+        for i in d["DevicePositions"]:
             if len(i["Position_um"]) == 1:
                 positions.append(Position1d.fromPropertyMap(i))
             elif len(i["Position_um"]) == 2:
                 positions.append(Position2d.fromPropertyMap(i))
             else:
                 raise Exception("EEEEE")
-        return MultiStagePosition(label=d['Label'].value, defaultXYStage=d['DefaultXYStage'].value, defaultZStage=d['DefaultZStage'].value, stagePositions=positions)
+        return MultiStagePosition(
+            label=d["Label"].value,
+            defaultXYStage=d["DefaultXYStage"].value,
+            defaultZStage=d["DefaultZStage"].value,
+            stagePositions=positions,
+        )
 
     def getXYPosition(self):
         """Return the first `Position2d` saved in the `positions` list"""
         d1pos = [i for i in self.stagePositions if isinstance(i, Position2d)]
         return [i for i in d1pos if i.stageName == self.defaultXYStage][0]
-    
+
     def getZPosition(self):
         """Return the first `Position1d` saved in the positions` list. Returns `None` if no position is found."""
         try:
@@ -203,7 +217,7 @@ class MultiStagePosition:
             return [i for i in d1pos if i.stageName == self.defaultZStage][0]
         except IndexError:
             return None
-    
+
     def renameXYStage(self, label: str):
         """Change the name of the xy stage.
 
@@ -221,7 +235,9 @@ class MultiStagePosition:
         """
         return copy.deepcopy(self)
 
-    def __add__(self, other: Union[Position2d, MultiStagePosition, PositionList]) -> Union[MultiStagePosition, PositionList]:
+    def __add__(
+        self, other: Union[Position2d, MultiStagePosition, PositionList]
+    ) -> Union[MultiStagePosition, PositionList]:
         """Allow adding a position to another position or `PositionList`. Doesn't work on the Z-Axis but will sum together
         the X and Y axis values. Used to translate a position by an offset.
 
@@ -233,40 +249,50 @@ class MultiStagePosition:
             positions = copy.copy(self.stagePositions)
             positions.remove(self.getXYPosition())
             positions.append(newPos)
-            return MultiStagePosition(self.label, self.defaultXYStage, self.defaultZStage, positions=positions)
+            return MultiStagePosition(
+                self.label, self.defaultXYStage, self.defaultZStage, positions=positions
+            )
         elif isinstance(other, MultiStagePosition):
             return self.__add__(other.getXYPosition())
         elif isinstance(other, PositionList):
             return other.__add__(self)
         else:
             raise NotImplementedError
-            
-    def __sub__(self, other: Union[Position2d, MultiStagePosition, PositionList]) -> Union[MultiStagePosition, PositionList]:
+
+    def __sub__(
+        self, other: Union[Position2d, MultiStagePosition, PositionList]
+    ) -> Union[MultiStagePosition, PositionList]:
         """See the documentation for __add__"""
         if isinstance(other, Position2d):
             newPos = self.getXYPosition().__sub__(other)
             positions = copy.copy(self.stagePositions)
             positions.remove(self.getXYPosition())
             positions.append(newPos)
-            return MultiStagePosition(self.label, self.defaultXYStage, self.defaultZStage, positions=positions)
+            return MultiStagePosition(
+                self.label, self.defaultXYStage, self.defaultZStage, positions=positions
+            )
         elif isinstance(other, MultiStagePosition):
             self.__sub__(other.getXYPosition())
         elif isinstance(other, PositionList):
             return other.copy().mirrorX().mirrorY().__add__(self)  # a-b == -b + a
         else:
             raise NotImplementedError
-    
+
     def __eq__(self, other: MultiStagePosition):
         """Returns True if the stage names and stage coordinates are equivalent."""
-        return all([self.defaultXYStage == other.defaultXYStage,
-                    self.defaultZStage == other.defaultZStage,
-                    self.getXYPosition() == other.getXYPosition(),
-                    self.getZPosition() == other.getZPosition()])
-    
+        return all(
+            [
+                self.defaultXYStage == other.defaultXYStage,
+                self.defaultZStage == other.defaultZStage,
+                self.getXYPosition() == other.getXYPosition(),
+                self.getZPosition() == other.getZPosition(),
+            ]
+        )
+
     def __repr__(self):
         s = f"MultiStagePosition({self.label}, "
         for i in self.stagePositions:
-            s += '\n\t' + i.__repr__()
+            s += "\n\t" + i.__repr__()
         return s
 
 
@@ -289,7 +315,7 @@ class PositionList:
     @staticmethod
     def fromDict(d: dict):
         p = []
-        for i in d['positions']:
+        for i in d["positions"]:
             p.append(MultiStagePosition.fromDict(i))
         return PositionList(p)
 
@@ -306,9 +332,9 @@ class PositionList:
     def mirrorY(self) -> PositionList:
         """Invert all y coordinates
 
-         Returns:
-             A reference to this object.
-         """
+        Returns:
+            A reference to this object.
+        """
         for i in self.positions:
             i.getXYPosition().mirrorY()
         return self
@@ -334,7 +360,9 @@ class PositionList:
         """Attempt to load a PositionList from a PropertyMap. May throw an exception."""
         if isinstance(pmap, PropertyMap):
             if "StagePositions" in pmap:
-                return PositionList([MultiStagePosition.fromPropertyMap(i) for i in pmap["StagePositions"]])
+                return PositionList(
+                    [MultiStagePosition.fromPropertyMap(i) for i in pmap["StagePositions"]]
+                )
         raise Exception("JsonParseException")
 
     @classmethod
@@ -343,9 +371,11 @@ class PositionList:
 
     def toPropertyMap(self) -> PropertyMap:
         """Returns the position list as a PropertyMap that is formatted just like a `PropertyMap` from Micro-Manager."""
-        pmap = PropertyMap({"StagePositions": PropertyMapArray([i.toPropertyMap() for i in self.positions])})
+        pmap = PropertyMap(
+            {"StagePositions": PropertyMapArray([i.toPropertyMap() for i in self.positions])}
+        )
         return pmap
-    
+
     def getAffineTransform(self, otherList: PositionList) -> np.ndarray:
         """
         Calculate the partial affine transformation between this position list and another position list. Both position lists must have the same length
@@ -365,6 +395,7 @@ class PositionList:
             newPos = origPos.applyAffineTransform(t)
         """
         import cv2
+
         assert len(otherList) == len(self)
         selfXY = [pos.getXYPosition() for pos in self.positions]
         otherXY = [pos.getXYPosition() for pos in otherList.positions]
@@ -380,25 +411,26 @@ class PositionList:
         """
 
         import cv2
+
         assert isinstance(t, np.ndarray)
         assert t.shape == (2, 3)
         selfXY = [pos.getXYPosition() for pos in self.positions]
         selfArr = np.array([[pos.x, pos.y] for pos in selfXY], dtype=np.float32)
-        selfArr = selfArr[None, :, :] # This is needed for opencv transform to work
+        selfArr = selfArr[None, :, :]  # This is needed for opencv transform to work
         ret = cv2.transform(selfArr, t)
         positions = []
         for i in range(len(self)):
             pos = copy.deepcopy(self[i].getXYPosition())
             pos.x = ret[0, i, 0]
             pos.y = ret[0, i, 1]
-            positions.append(MultiStagePosition(self[i].label, self[i].defaultXYStage, '', [pos]))
+            positions.append(MultiStagePosition(self[i].label, self[i].defaultXYStage, "", [pos]))
         return PositionList(positions)
 
     def __repr__(self):
         s = "PositionList(\n["
         for i in self.positions:
-            s += str(i) + '\n'
-        s += '])'
+            s += str(i) + "\n"
+        s += "])"
         return s
 
     def __add__(self, other: Union[Position2d, MultiStagePosition]) -> PositionList:
@@ -424,30 +456,38 @@ class PositionList:
         return self.positions[idx]
 
     def __eq__(self, other: PositionList):
-        return all([len(self) == len(other)] +
-                   [self[i] == other[i] for i in range(len(self))])
+        return all([len(self) == len(other)] + [self[i] == other[i] for i in range(len(self))])
 
     def plot(self):
         """Open a matplotlib plot showing the positions contained in this list."""
         fig, ax = plt.subplots()
-        annot = ax.annotate("", xy=(0, 0), xytext=(20, 20), textcoords="offset points",
-                            bbox=dict(boxstyle="round", fc="w"),
-                            arrowprops=dict(arrowstyle="->"))
+        annot = ax.annotate(
+            "",
+            xy=(0, 0),
+            xytext=(20, 20),
+            textcoords="offset points",
+            bbox=dict(boxstyle="round", fc="w"),
+            arrowprops=dict(arrowstyle="->"),
+        )
         annot.set_visible(False)
         ax.set_xlabel("x")
-        ax.set_ylabel('y')
-        ax.set_aspect('equal')
+        ax.set_ylabel("y")
+        ax.set_aspect("equal")
         cmap = mpl.cm.get_cmap("gist_rainbow")
         colors = [cmap(i) for i in np.linspace(0, 1, num=len(self.positions))]
         names = [pos.label for pos in self.positions]
-        sc = plt.scatter([pos.getXYPosition().x for pos in self.positions], [pos.getXYPosition().y for pos in self.positions],
-                         c=[colors[i] for i in range(len(self.positions))])
+        sc = plt.scatter(
+            [pos.getXYPosition().x for pos in self.positions],
+            [pos.getXYPosition().y for pos in self.positions],
+            c=[colors[i] for i in range(len(self.positions))],
+        )
 
         def update_annot(ind):
             pos = sc.get_offsets()[ind["ind"][0]]
             annot.xy = pos
-            text = "{}, {}".format(" ".join(list(map(str, ind["ind"]))),
-                                   " ".join([names[n] for n in ind["ind"]]))
+            text = "{}, {}".format(
+                " ".join(list(map(str, ind["ind"]))), " ".join([names[n] for n in ind["ind"]])
+            )
             annot.set_text(text)
             #            annot.get_bbox_patch().set_facecolor(cmap(norm(c[ind["ind"][0]])))
             annot.get_bbox_patch().set_alpha(0.4)
@@ -468,11 +508,9 @@ class PositionList:
         fig.canvas.mpl_connect("motion_notify_event", hover)
 
 
-
-
-if __name__ == '__main__':
-    path1 = r'PositionList.pos'
-    path2 = r'PositionListOut.pos'
+if __name__ == "__main__":
+    path1 = r"PositionList.pos"
+    path2 = r"PositionListOut.pos"
 
     ## Test that loading, mirroring, and then saving results in no changes to the position list.
     p = PropertyMap.loadFromFile(path1)
@@ -483,7 +521,6 @@ if __name__ == '__main__':
     with open(path1) as f1, open(path2) as f2:
         assert f1.read() == f2.read()
 
-
     def generateList(data: np.ndarray):
         """Example function to create a brand new position list in python."""
         assert isinstance(data, np.ndarray)
@@ -491,14 +528,6 @@ if __name__ == '__main__':
         assert data.shape[1] == 2
         positions = []
         for n, i in enumerate(data):
-            positions.append(Position2d(*i, 'TIXYDrive', f'Cell{n + 1}'))
+            positions.append(Position2d(*i, "TIXYDrive", f"Cell{n + 1}"))
         plist = PositionList(positions)
         return plist
-
-
-
-
-
-
-
-
